"""empty message

Revision ID: 4e1eb8b4e7f2
Revises: 976c33d240d0
Create Date: 2023-02-23 16:26:11.378536

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "4e1eb8b4e7f2"
down_revision = "976c33d240d0"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "uniq_cons_site_start_end", "generation", ["site_uuid", "start_utc", "end_utc"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("uniq_cons_site_start_end", "generation", type_="unique")
    # ### end Alembic commands ###



"""Python SDK for reading/writing to/from pvsite database."""

from .connection import DatabaseConnection
from .sqlmodels import ClientSQL, ForecastSQL, ForecastValueSQL, GenerationSQL, SiteSQL, StatusSQL



"""Functions for reading from latest_forecast_values table."""

import datetime as dt
import uuid
from typing import Dict, List, Optional

from sqlalchemy.orm import Session

from pvsite_datamodel.sqlmodels import ForecastValueSQL


def get_forecast_values_by_site_latest(
    session: Session,
    site_uuids: List[uuid.UUID],
    start_utc: Optional[dt.datetime] = None,
) -> Dict[uuid.UUID, List[ForecastValueSQL]]:
    """Get the forecast values by input sites, get the lastest value.

    This reads the ForecastValueSQL table

    :param session: The sqlalchemy database session
    :param site_uuids: list of site_uuids for which to fetch latest forecast values
    :param start_utc: filters on forecast values target_time >= start_utc
    """
    query = session.query(ForecastValueSQL).distinct(
        ForecastValueSQL.site_uuid, ForecastValueSQL.start_utc
    )

    if start_utc is not None:
        query = query.filter(ForecastValueSQL.start_utc >= start_utc)

    # Filter the query on the desired sites
    query = query.filter(ForecastValueSQL.site_uuid.in_(site_uuids))

    query.order_by(
        ForecastValueSQL.site_uuid,
        ForecastValueSQL.start_utc,
        ForecastValueSQL.created_utc.desc(),
    )

    # query results
    forecast_values = query.all()

    output_dict: Dict[uuid.UUID, List[ForecastValueSQL]] = {}

    for site_uuid in site_uuids:
        site_latest_forecast_values: List[ForecastValueSQL] = [
            fv for fv in forecast_values if fv.forecast.site_uuid == site_uuid
        ]

        output_dict[site_uuid] = site_latest_forecast_values

    return output_dict



"""
Functions for reading from the PVSite database
"""

from .generation import get_pv_generation_by_client, get_pv_generation_by_sites
from .latest_forecast_values import get_forecast_values_by_site_latest
from .site import (
    get_all_sites,
    get_site_by_client_site_id,
    get_site_by_client_site_name,
    get_site_by_uuid,
)
from .status import get_latest_status



"""Database Connection class."""
import logging

from sqlalchemy import create_engine
from sqlalchemy.engine import URL
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm.session import Session

logger = logging.getLogger(__name__)


class DatabaseConnection:
    """Database connection class."""

    def __init__(self, url: URL | str, echo: bool = True):
        """Set up database connection.

        :param url: the database url, used for connecting
        :param echo: whether to echo
        """
        self.url = url
        self.engine = create_engine(self.url, echo=echo)
        self.Session = sessionmaker(bind=self.engine)
        if self.url is None:
            raise Exception("Need to set url for database connection")

    def get_session(self) -> Session:
        """Get sqlalchemy session."""
        return self.Session()



"""empty message

Revision ID: 976c33d240d0
Revises: 
Create Date: 2023-02-23 14:12:53.160163

"""
import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "976c33d240d0"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "clients",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("client_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("client_name", sa.String(length=255), nullable=False),
        sa.PrimaryKeyConstraint("client_uuid"),
    )
    op.create_table(
        "status",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("status_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("status", sa.String(length=255), nullable=True),
        sa.Column("message", sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint("status_uuid"),
    )
    op.create_table(
        "sites",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("site_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("client_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("client_site_id", sa.Integer(), nullable=True),
        sa.Column("client_site_name", sa.String(length=255), nullable=True),
        sa.Column("region", sa.String(length=255), nullable=True),
        sa.Column("dno", sa.String(length=255), nullable=True),
        sa.Column("gsp", sa.String(length=255), nullable=True),
        sa.Column("orientation", sa.Float(), nullable=True),
        sa.Column("tilt", sa.Float(), nullable=True),
        sa.Column("latitude", sa.Float(), nullable=True),
        sa.Column("longitude", sa.Float(), nullable=True),
        sa.Column("capacity_kw", sa.Float(), nullable=True),
        sa.Column("ml_id", sa.Integer(), autoincrement=True, nullable=False),
        sa.ForeignKeyConstraint(
            ["client_uuid"],
            ["clients.client_uuid"],
        ),
        sa.PrimaryKeyConstraint("site_uuid"),
        sa.UniqueConstraint("client_site_id", "client_uuid", name="idx_client"),
        sa.UniqueConstraint("ml_id"),
    )
    op.create_index(op.f("ix_sites_client_site_id"), "sites", ["client_site_id"], unique=False)
    op.create_index(op.f("ix_sites_client_site_name"), "sites", ["client_site_name"], unique=False)
    op.create_table(
        "forecasts",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("forecast_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("site_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("forecast_version", sa.String(length=32), nullable=False),
        sa.ForeignKeyConstraint(
            ["site_uuid"],
            ["sites.site_uuid"],
        ),
        sa.PrimaryKeyConstraint("forecast_uuid"),
    )
    op.create_index(op.f("ix_forecasts_site_uuid"), "forecasts", ["site_uuid"], unique=False)
    op.create_table(
        "generation",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("generation_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("site_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("generation_power_kw", sa.Float(), nullable=False),
        sa.Column("start_utc", sa.DateTime(), nullable=False),
        sa.Column("end_utc", sa.DateTime(), nullable=False),
        sa.ForeignKeyConstraint(
            ["site_uuid"],
            ["sites.site_uuid"],
        ),
        sa.PrimaryKeyConstraint("generation_uuid"),
    )
    op.create_index(op.f("ix_generation_site_uuid"), "generation", ["site_uuid"], unique=False)
    op.create_index(op.f("ix_generation_start_utc"), "generation", ["start_utc"], unique=False)
    op.create_table(
        "forecast_values",
        sa.Column("created_utc", sa.DateTime(), nullable=True),
        sa.Column("forecast_value_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("start_utc", sa.DateTime(), nullable=False),
        sa.Column("end_utc", sa.DateTime(), nullable=False),
        sa.Column("forecast_power_kw", sa.Float(), nullable=False),
        sa.Column("forecast_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("site_uuid", postgresql.UUID(as_uuid=True), nullable=False),
        sa.ForeignKeyConstraint(
            ["forecast_uuid"],
            ["forecasts.forecast_uuid"],
        ),
        sa.ForeignKeyConstraint(
            ["site_uuid"],
            ["sites.site_uuid"],
        ),
        sa.PrimaryKeyConstraint("forecast_value_uuid"),
    )
    op.create_index(
        op.f("ix_forecast_values_forecast_uuid"), "forecast_values", ["forecast_uuid"], unique=False
    )
    op.create_index(
        op.f("ix_forecast_values_site_uuid"), "forecast_values", ["site_uuid"], unique=False
    )
    op.create_index(
        op.f("ix_forecast_values_start_utc"), "forecast_values", ["start_utc"], unique=False
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_forecast_values_start_utc"), table_name="forecast_values")
    op.drop_index(op.f("ix_forecast_values_site_uuid"), table_name="forecast_values")
    op.drop_index(op.f("ix_forecast_values_forecast_uuid"), table_name="forecast_values")
    op.drop_table("forecast_values")
    op.drop_index(op.f("ix_generation_start_utc"), table_name="generation")
    op.drop_index(op.f("ix_generation_site_uuid"), table_name="generation")
    op.drop_table("generation")
    op.drop_index(op.f("ix_forecasts_site_uuid"), table_name="forecasts")
    op.drop_table("forecasts")
    op.drop_index(op.f("ix_sites_client_site_name"), table_name="sites")
    op.drop_index(op.f("ix_sites_client_site_id"), table_name="sites")
    op.drop_table("sites")
    op.drop_table("status")
    op.drop_table("clients")
    # ### end Alembic commands ###



"""Run migrations on Forecast Database."""
import os
from logging.config import fileConfig

from alembic import context

from pvsite_datamodel.connection import DatabaseConnection
from pvsite_datamodel.sqlmodels import Base

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
fileConfig(config.config_file_name)

# add your model's MetaData object here
# for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = Base.metadata


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """

    connection = DatabaseConnection(os.getenv("DB_URL"))
    connectable = connection.engine

    with connectable.connect() as connection:
        context.configure(connection=connection, target_metadata=Base.metadata)

        with context.begin_transaction():
            context.run_migrations()


# need to have this code running when called
run_migrations_online()
